import os
from logging import Logger
import logging
import configparser

from playoff import Playoff
from dotenv import load_dotenv


class ParameterException(Exception):
    """Class that define an exception for parameters methods"""
    pass


class Constant(object):
    """Class that define some useful costant"""
    config = configparser.ConfigParser()
    file_name = "settings.ini"
    config.read(file_name)

    VERSION = config.get("constant", "Version")
    PLAYER_ID = config.get("constant", "Player_id")
    CYCLE = config.get("constant", "Cycle")

    BIG_NUMBER = 10 ** 10
    TOTAL = "total"

    SCORES = "/scores/"

    ADMIN_ROOT = "/admin/"
    ADMIN_PLAYERS = "/admin/players/"
    ADMIN_TEAMS = "/admin/teams/"

    DESIGN_TEAMS = "/design/versions/" + VERSION + "/teams/"
    DESIGN_ACTIONS = "/design/versions/" + VERSION + "/actions/"
    DESIGN_METRICS = "/design/versions/" + VERSION + "/metrics/"
    DESIGN_LEADERBOARDS = "/design/versions/" + VERSION + "/leaderboards/"
    DESIGN_RULES = "/design/versions/" + VERSION + "/rules/"
    DESIGN_DEPLOY = "/design/versions/" + VERSION + "/deploy/"

    RUNTIME_ACTION = "/runtime/actions/"
    RUNTIME_LEADERBOARDS = "/runtime/leaderboards/"


class MigrationLogger:
    __instance: Logger = None

    @staticmethod
    def get_instance():
        if MigrationLogger.__instance is None:
            MigrationLogger()
        return MigrationLogger.__instance

    def __init__(self):
        """ Virtually private constructor. """
        if MigrationLogger.__instance is not None:
            raise Exception("This class is a singleton!")
        else:
            config = configparser.ConfigParser()
            file_name = "settings.ini"
            config.read(file_name)

            logger_level = config.get("logger", "Level").upper()

            MigrationLogger.__instance = logging.getLogger("migration_logger")
            MigrationLogger.__instance.setLevel(logging.DEBUG)
            ch = logging.FileHandler(filename="migration.log", mode="w")
            ch.setLevel(logger_level)
            formatter = logging.Formatter(
                "%(asctime)s - %(levelname)s - %(message)s",
                "%m/%d/%Y %I:%M:%S %p")
            ch.setFormatter(formatter)
            MigrationLogger.__instance.addHandler(ch)


class Utility(object):
    """Class that define some useful methods"""

    @staticmethod
    def get_number_pages(number):
        """Return number of pages

        :param number: number of items to paginate
        :raise ParameterException: if parameter is negative

        Pages refers to pagination
        Every page has 100 item, so:
        number = 100 -> return 1
        number = 101 -> return 2
        """
        logger = MigrationLogger.get_instance()
        logger.debug("get_number_pages called")

        if number < 0:
            raise ParameterException("Parameter can't be negative")

        division_res = int(number / 100)

        return division_res if number % 100 == 0 else division_res + 1

    @staticmethod
    def raise_empty_parameter_exception(parameters):
        """Raise an exception if one parameter is empty

        :param list parameters: list of parameters to check
        :raise ParameterException
        """
        logger = MigrationLogger.get_instance()

        for par in parameters:
            if not isinstance(par, int):
                if not par:
                    logger.warning("parameters: " + str(parameters))

                    raise ParameterException("Parameter can't be empty")

    @staticmethod
    def get_playoff_client(client_id, client_secret, hostname):
        """Return Playoff game instance given his client id , client secret and
        hostname

        :param client_id: Playoff game client id
        :param client_secret: Playoff game client secret
        :param hostname: hostname
        :return: Playoff game instance
        """
        Utility.raise_empty_parameter_exception([client_id, client_secret,
                                                 hostname])

        logger = MigrationLogger.get_instance()
        logger.info("A new playoff client will be created...")

        from pathlib import Path
        env_path = Path('.') / '.env'
        load_dotenv(dotenv_path=env_path)

        return Playoff(
            client_id=os.environ[client_id],
            client_secret=os.environ[client_secret],
            type='client',
            allow_unsecure=True,
            hostname=os.environ[hostname]
        )



from .utility import MigrationLogger, Constant, Utility
from .design_handler import GetPlayoffDesign

from playoff import Playoff


class GetPlayoffData(object):
    """Class that make GET call via Playoff client to retrieve data from
    the Playoff game
    """

    def __init__(self, game: Playoff):
        self.game = game
        self.logger = MigrationLogger.get_instance()

    # ==============
    # COUNT METHODS
    # ==============

    def get_team_count(self):
        """Return number of teams in the game"""
        self.logger.debug("returning number of teams")

        return self.game.get(Constant.ADMIN_TEAMS, {})[Constant.TOTAL]

    def get_players_count(self):
        """Returns number of players in the game"""
        self.logger.debug("returning number of players")

        return self.game.get(Constant.ADMIN_PLAYERS, {})[Constant.TOTAL]

    def get_players_count_in_team(self, team_id):
        """Return number of players of the chosen team

        :param str team_id: containing id of a team
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([team_id])

        self.logger.debug("returning number of players in team: " + team_id)

        return self.game.get(Constant.ADMIN_TEAMS + team_id +
                             '/members', {})[Constant.TOTAL]

    # ==============
    # INFO METHODS
    # ==============

    def get_game_id(self):
        """ Returns game id of the chosen game """
        self.logger.debug("returning game_id")

        return self.game.get(Constant.ADMIN_ROOT)["game"]["id"]

    def get_teams_by_id(self):
        """Returns a list of teams id"""
        teams_id = []
        number_teams = self.get_team_count()
        number_pages = Utility.get_number_pages(number_teams)

        self.logger.info("preparing list of teams id")

        for count in range(number_pages):

            teams = self.game.get(Constant.ADMIN_TEAMS,
                                  {"skip": str(count * 100), "limit": "100"})

            for team in teams['data']:
                teams_id.append(team['id'])

                self.logger.debug(team['id'] + " added to list")

        self.logger.info("returning list of teams id")

        return teams_id

    def get_team_info(self, team_id):
        """Return information of the chosen team

        :param str team_id: containing id of a team
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([team_id])

        self.logger.debug("returning info of team: " + team_id)

        return self.game.get(Constant.ADMIN_TEAMS + team_id, {})

    def get_players_by_id(self):
        """Return a list of players id"""
        players_id = []
        number_players = self.get_players_count()
        number_pages = Utility.get_number_pages(number_players)

        self.logger.info("preparing list of players id")

        for count in range(number_pages):
            players = self.game.get(Constant.ADMIN_PLAYERS,
                                    {"skip": str(count * 100), "limit": "100"})

            for player in players['data']:
                players_id.append(player['id'])

                self.logger.debug(player['id'] + "added to list")

        self.logger.info("returning list of players id")

        return players_id

    def get_player_profile(self, player_id):
        """Return profile data of the chosen player

        :param str player_id: containing id of a player
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([player_id])

        self.logger.debug("returning info of player: " + player_id)

        return self.game.get(Constant.ADMIN_PLAYERS + player_id, {})

    def get_player_feed(self, player_id):
        """Return a list of feed of the chosen player

        :param str player_id: player id
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([player_id])

        self.logger.debug("returning feed of player: " + player_id)

        player_feed = self.game.get(Constant.ADMIN_PLAYERS + player_id +
                                    "/activity", {"start": "0"})

        if player_feed is None:
            return []

        return player_feed

    def get_leaderboard(self, leaderboard_id):
        """Return chosen leaderboard

        :param str leaderboard_id: chosen leaderboard
        :raise: ParameterException: when parameter is empty
        """
        Utility.raise_empty_parameter_exception([leaderboard_id])

        self.logger.debug("returning leaderboard: " + leaderboard_id)

        data = {
            "player_id": Constant.PLAYER_ID,
            "cycle": Constant.CYCLE,
            "limit": Constant.BIG_NUMBER
        }

        return self.game.get(Constant.RUNTIME_LEADERBOARDS + leaderboard_id,
                             data)

    def get_metric_scores(self, player_id):
        """
        Return a list of all scores of the chosen player

        :param player_id: player id
        :return: list containing player's scores
        """
        Utility.raise_empty_parameter_exception([player_id])

        player_profile = self.get_player_profile(player_id)

        self.logger.info("returning player " + player_id + " scores")

        return player_profile['scores']

    def get_single_metric_score(self, player_id, metric_id):
        """
        Return score of the chosen player in the chosen metric
        :param player_id: player id
        :param metric_id: metric to get score
        :raise exception: if metric_id doesn't exists in the game
        :return: score of the player in the given metric
        """
        Utility.raise_empty_parameter_exception([player_id, metric_id])

        player_scores = self.get_metric_scores(player_id)

        self.logger.info("returning metric " + metric_id)

        for score in player_scores:
            if score['metric']['id'] == metric_id:
                return score

        self.logger.warning("metric " + metric_id + " wasn't found")
        raise Exception("no metric with given name was found")


class PostPlayoffData(object):
    """Class that make POST call via Playoff client to create instances
    in the Playoff game
    """

    def __init__(self, game: Playoff):
        self.game = game
        self.logger = MigrationLogger.get_instance()

    def create_team(self, team_data):
        """Create a team

        :param dict team_data: team info necessary to create a team
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([team_data])

        self.logger.debug("creating team " + team_data['id'])

        self.game.post(Constant.ADMIN_TEAMS, {}, team_data)

        self.logger.debug("team created")

    def create_player(self, player_data):
        """Create a player

        :param dict player_data: player info necessary to create a player
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([player_data])

        self.logger.debug("creating player " + player_data['id'])

        self.game.post(Constant.ADMIN_PLAYERS, {}, player_data)

        self.logger.debug("player created")

    def join_team(self, team_id, data):
        """Join a team

        :param str team_id: team id to join
        :param dict data: data necessary to join a team
        :raise ParameterException: if a parameter is empty
        """
        Utility.raise_empty_parameter_exception([team_id, data])

        self.logger.debug("join team " + team_id)

        self.game.post(Constant.ADMIN_TEAMS + team_id + "/join", {}, data)

        self.logger.debug("team joined")

    def take_action(self, action_id, player_id, data):
        """Take an action

        :param str action_id: action id to take
        :param dict player_id: player id that take action
        :param dict data: data necessary to take action
        :raise ParameterException: if a parameter is empty
        """
        Utility.raise_empty_parameter_exception([action_id, player_id, data])

        self.logger.debug("taking action " + action_id + " by " +
                          player_id['player_id'])

        self.game.post(Constant.RUNTIME_ACTION + action_id + "/play",
                       player_id, data)

        self.logger.debug("action taken")


class DeletePlayoffData(object):
    """Class that make DELETE call via Playoff client to erase data
    from the Playoff game
    """

    def __init__(self, game: Playoff):
        self.game = game
        self.data_getter = GetPlayoffData(game)
        self.logger = MigrationLogger.get_instance()

    def delete_single_team(self, team_id):
        """Delete chosen team

        :param str team_id: team id to destroy
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([team_id])

        self.logger.debug("team " + team_id + " will be deleted")

        self.game.delete(Constant.ADMIN_TEAMS + team_id, {})

        self.logger.debug("team deleted")

    def delete_single_player(self, player_id):
        """Delete chosen player

        :param str player_id: player id to destroy
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([player_id])

        self.logger.debug("player " + player_id + " will be deleted")

        self.game.delete(Constant.ADMIN_PLAYERS + player_id, {})

        self.logger.debug("player deleted")

    def delete_teams(self):
        """Delete all teams"""
        teams_by_id = self.data_getter.get_teams_by_id()
        teams_count = str(len(teams_by_id))

        self.logger.info(teams_count + " teams will be deleted")
        index = 0

        for team in teams_by_id:
            self.delete_single_team(team)

            index += 1
            self.logger.debug("team " + str(index) + " of " + teams_count +
                              " deleted")

        self.logger.info("teams deleted")

    def delete_players(self):
        """Delete all players"""
        players_by_id = self.data_getter.get_players_by_id()
        players_count = str(len(players_by_id))

        self.logger.info(players_count + " players will be deleted")
        index = 0

        for player in players_by_id:
            self.delete_single_player(player)

            index += 1
            self.logger.debug("player " + str(index) + " of " + players_count +
                              " deleted")

        self.logger.info("players deleted")

    def delete_all_data(self):
        """Delete all data from the game"""
        self.logger.info("deleting data")

        self.delete_players()
        self.delete_teams()

        self.logger.info("data deleted")


class PatchPlayoffData(object):
    """
    Class that make PATCH call via Playoff client to modify data in the
    Playoff game
    """
    def __init__(self, game: Playoff):
        self.game = game
        self.design_getter = GetPlayoffDesign(game)
        self.logger = MigrationLogger.get_instance()

    @staticmethod
    def state_metric_parser(metric_constraints):
        """
        Return dict containing item name as a key and id as a value
        :param metric_constraints: constraints to extract name and id
        """
        Utility.raise_empty_parameter_exception([metric_constraints])

        set_metric_dict = {}

        for item in metric_constraints['states']:
            key = item['name']
            value = item['id']

            set_metric_dict.update({key: value})

        return set_metric_dict

    @staticmethod
    def set_metric_parser(metric_constraints):
        """
        Return dict containing item name as a key and id as a value
        :param metric_constraints: constraints to extract name and id
        """
        Utility.raise_empty_parameter_exception([metric_constraints])

        set_metric_dict = {}

        for item in metric_constraints['items']:
            key = item['name']
            value = item['id']

            set_metric_dict.update({key: value})

        return set_metric_dict

    def value_parser(self, metric):
        """
        Return value formatted right to be used in patch method
        :param metric: metric from retrieve information
        """
        Utility.raise_empty_parameter_exception([metric])

        metric_id = metric['metric']['id']
        metric_type = metric['metric']['type']
        value = metric['value']

        metric_design = self.design_getter.get_single_metric_design(metric_id)
        metric_constraints = metric_design['constraints']

        if metric_type == "point":
            return value
        elif metric_type == "set":
            set_parser = self.set_metric_parser(metric_constraints)
            cloned_value = {}

            for score in value:
                key = set_parser[score['name']]
                value = score['count']

                cloned_value.update({key: value})

            return cloned_value
        elif metric_type == "state":
            state_parser = self.state_metric_parser(metric_constraints)

            return state_parser[value['name']]

    def json_metric_parser(self, metric):
        """
        Return json data accepted to json schema for call patch method

        :param metric: metric data to create dict/json
        :return: dict/json to use in patch_player_score
        """
        Utility.raise_empty_parameter_exception([metric])

        metric_id = metric['metric']['id']
        metric_type = metric['metric']['type']
        value = self.value_parser(metric)

        data = {
            "rewards": [{
                "metric": {
                    "id": metric_id,
                    "type": metric_type
                },
                "verb": "set",
                "value": value
            }]
        }

        self.logger.debug("returning json score")
        self.logger.debug(data)

        return data

    def patch_player_score(self, player_id, data):
        """
        Update a player score

        :param player_id: player id to update score
        :param data: data necessary to update a score
        """
        Utility.raise_empty_parameter_exception([player_id, data])

        self.logger.debug("updating score of player " + player_id)
        self.logger.debug("data given")
        self.logger.debug(data)

        self.game.patch(Constant.ADMIN_PLAYERS + player_id + Constant.SCORES,
                        {"player_id": player_id}, data)

        self.logger.debug("scores updated")

    def update_metric(self, player_id, metric_value):
        """
        Update a score of the given player in the given metric

        :param player_id: player id
        :param metric_value: metric data
        """
        Utility.raise_empty_parameter_exception([player_id, metric_value])

        metric_id = metric_value['metric']['id']

        self.logger.info("updating metric " + metric_id + " of player " +
                         player_id)

        # making json data for patch call
        data = self.json_metric_parser(metric_value)
        self.patch_player_score(player_id, data)



from playoff_core.data_handler import GetPlayoffData, PostPlayoffData, \
    DeletePlayoffData, PatchPlayoffData
from playoff_core.design_handler import GetPlayoffDesign, PostPlayoffDesign, \
    DeletePlayoffDesign
from playoff_core.import_export_playoff import ImportData, ImportDesign, \
    ExportRawDesign, ExportRawData, ExportData, ExportDesign
from playoff_core.utility import Utility

name = "playoff_core"



from .utility import MigrationLogger, Constant, Utility

from playoff import Playoff


class GetPlayoffDesign(object):
    """Class that make GET call via Playoff client to retrieve design from
    the Playoff game
    """

    def __init__(self, game: Playoff):
        self.game = game
        self.logger = MigrationLogger.get_instance()

    def get_teams_design(self):
        """Return a list containing all teams design"""
        self.logger.debug("calling playoff for teams design")

        return self.game.get(Constant.DESIGN_TEAMS, {})

    def get_single_team_design(self, team_id):
        """Return design of the chosen team

        :param str team_id: id of the team
        """
        Utility.raise_empty_parameter_exception([team_id])

        self.logger.debug("returning " + team_id + " design")

        return self.game.get(Constant.DESIGN_TEAMS + team_id, {})

    def get_metrics_design(self):
        """Return a list containing all metrics design"""
        self.logger.debug("calling playoff for metrics design")

        return self.game.get(Constant.DESIGN_METRICS, {})

    def get_single_metric_design(self, metric_id):
        """Return design of the chosen metric

        :param str metric_id: id of metric
        """
        Utility.raise_empty_parameter_exception([metric_id])

        self.logger.debug("returning " + metric_id + " design")

        return self.game.get(Constant.DESIGN_METRICS + metric_id, {})

    def get_actions_design(self):
        """Return a list containing all actions design"""
        self.logger.debug("calling playoff for actions design")

        return self.game.get(Constant.DESIGN_ACTIONS, {})

    def get_single_action_design(self, action_id):
        """Return design of the chosen action

        :param str action_id: id of action
        """
        Utility.raise_empty_parameter_exception([action_id])

        self.logger.debug("returning " + action_id + " design")

        return self.game.get(Constant.DESIGN_ACTIONS + action_id, {})

    def get_leaderboards_design(self):
        """Return a list of dict containing leaderboards design id and name"""
        self.logger.debug("calling playoff for leaderboards design")

        return self.game.get(Constant.DESIGN_LEADERBOARDS, {})

    def get_single_leaderboard_design(self, leaderboard_id):
        """Return design of the chosen leaderboard

        :param str leaderboard_id: id of leaderboard
        """
        Utility.raise_empty_parameter_exception([leaderboard_id])

        self.logger.debug("returning " + leaderboard_id + " design")

        return self.game.get(Constant.DESIGN_LEADERBOARDS + leaderboard_id, {})

    def get_rules_design(self):
        """Return a list of dict containing rules design id and name"""
        self.logger.debug("calling playoff for rules design")

        return self.game.get(Constant.DESIGN_RULES, {})

    def get_single_rule_design(self, rule_id):
        """Return design of the chosen rule

        :param str rule_id: id of rule
        """
        Utility.raise_empty_parameter_exception([rule_id])

        self.logger.debug("returning " + rule_id + " design")

        return self.game.get(Constant.DESIGN_RULES + rule_id, {})


class PostPlayoffDesign(object):
    """Class that make POST call via Playoff client to create design
    in the Playoff game
    """
    def __init__(self, game: Playoff):
        self.game = game
        self.logger = MigrationLogger.get_instance()

    def create_team_design(self, design_data):
        """Create a team design

        :param dict design_data: info necessary to create a team design
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([design_data])

        self.logger.debug("creating team design")

        self.game.post(Constant.DESIGN_TEAMS, {}, design_data)

        self.logger.debug("team design created")

    def create_metric_design(self, design_data):
        """Create a metric design

        :param dict design_data: info necessary to create a metric design
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([design_data])

        self.logger.debug("creating metric design")

        self.game.post(Constant.DESIGN_METRICS, {}, design_data)

        self.logger.debug("team design created")

    def create_action_design(self, design_data):
        """Create a action design

        :param dict design_data: info necessary to create an action design
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([design_data])

        self.logger.debug("creating action design")

        self.game.post(Constant.DESIGN_ACTIONS, {}, design_data)

        self.logger.debug("action design created")

    def create_leaderboard_design(self, design_data):
        """Create a leaderboard design

        :param dict design_data: info necessary to create an leaderboard design
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([design_data])

        self.logger.debug("creating leaderboard design")

        self.game.post(Constant.DESIGN_LEADERBOARDS, {}, design_data)

        self.logger.debug("leaderboard design created")

    def create_rule_design(self, design_data):
        """Create a rule design

        :param dict design_data: info necessary to create an rule design
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([design_data])

        self.logger.debug("creating rule design")

        self.game.post(Constant.DESIGN_RULES, {}, design_data)

        self.logger.debug("rule design created")


class DeletePlayoffDesign(object):
    """Class that make DELETE call via Playoff client to erase design
    from the Playoff game
    """

    def __init__(self, game: Playoff):
        self.game = game
        self.design_getter = GetPlayoffDesign(game)
        self.logger = MigrationLogger.get_instance()

    def delete_single_team_design(self, team_id):
        """Delete chosen team_id from the game

        :param str team_id: team id to delete
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([team_id])

        self.logger.debug("deleting " + team_id + " design")

        self.game.delete(Constant.DESIGN_TEAMS + team_id, {})

    def delete_single_metric_design(self, metric_id):
        """Delete chosen team_id from the game

        :param str metric_id: team id to delete
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([metric_id])

        self.logger.debug("deleting " + metric_id + " design")

        self.game.delete(Constant.DESIGN_METRICS + metric_id, {})

    def delete_single_action_design(self, action_id):
        """Delete chosen team_id from the game

        :param str action_id: team id to delete
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([action_id])

        self.logger.debug("deleting " + action_id + " design")

        self.game.delete(Constant.DESIGN_ACTIONS + action_id, {})

    def delete_single_leaderboard_design(self, leaderboard_id):
        """Delete chosen team_id from the game

        :param str leaderboard_id: team id to delete
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([leaderboard_id])

        self.logger.debug("deleting " + leaderboard_id + " design")

        self.game.delete(Constant.DESIGN_LEADERBOARDS + leaderboard_id, {})

    def delete_single_rule_design(self, rule_id):
        """Delete chosen rule_id from the game

        :param str rule_id: rule id to delete
        :raise ParameterException: if parameter is empty
        """
        Utility.raise_empty_parameter_exception([rule_id])

        self.logger.debug("deleting " + rule_id + " design")

        self.game.delete(Constant.DESIGN_RULES + rule_id, {})

    def delete_teams_design(self):
        """Delete teams design"""
        teams_design = self.design_getter.get_teams_design()
        teams_count = str(len(teams_design))

        self.logger.info(teams_count + " teams design will be deleted")
        index = 0

        for team in teams_design:
            self.delete_single_team_design(team['id'])

            index += 1
            self.logger.debug("team " + str(index) + " of " + teams_count +
                              " deleted")

        self.logger.info("teams deleted")

    def delete_metrics_design(self):
        """Delete metrics design"""
        metrics_design = self.design_getter.get_metrics_design()
        metrics_count = str(len(metrics_design))

        self.logger.info(metrics_count + " metrics design will be deleted")
        index = 0

        for metric in metrics_design:
            self.delete_single_metric_design(metric['id'])

            index += 1
            self.logger.debug("metric " + str(index) + " of " + metrics_count +
                              " deleted")

        self.logger.info("metrics deleted")

    def delete_actions_design(self):
        """Delete actions design"""
        actions_design = self.design_getter.get_actions_design()
        actions_count = str(len(actions_design))

        self.logger.info(actions_count + " actions design will be deleted")
        index = 0

        for action in actions_design:
            self.delete_single_action_design(action['id'])

            index += 1
            self.logger.debug("action " + str(index) + " of " + actions_count +
                              " deleted")

        self.logger.info("actions deleted")

    def delete_leaderboards_design(self):
        """Delete leaderboards design"""
        leaderboards_design = self.design_getter.get_leaderboards_design()
        leaderboards_count = str(len(leaderboards_design))

        self.logger.info(leaderboards_count + " leaderboards design will be "
                                              "deleted")
        index = 0

        for leaderboard in leaderboards_design:
            self.delete_single_leaderboard_design(leaderboard['id'])

            index += 1
            self.logger.debug("leaderboard " + str(index) + " of " +
                              leaderboards_count + " deleted")

        self.logger.info("leaderboards deleted")

    def delete_rules_design(self):
        """Delete rules design"""
        rules_design = self.design_getter.get_rules_design()
        rules_count = str(len(rules_design))

        self.logger.info(rules_count + " rules design will be "
                                       "deleted")
        index = 0

        for rule in rules_design:
            self.delete_single_rule_design(rule['id'])

            index += 1
            self.logger.debug("rule " + str(index) + " of " +
                              rules_count + " deleted")

        self.logger.info("rules deleted")

    def delete_all_design(self):
        """Delete all design from the game"""
        self.logger.info("deleting all design")

        self.delete_leaderboards_design()
        self.delete_actions_design()
        self.delete_metrics_design()
        self.delete_teams_design()
        self.delete_rules_design()

        self.logger.info("all design deleted")



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="playoff_core",
    version="0.8.3",
    author="Mattia Bano",
    author_email="dev@officina.cc",
    description="Playoff toolbox",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/officina/playoff-core",
    packages=setuptools.find_packages(),
    install_requires=[
        "Playoff",
        "jwt",
        "python-dotenv"

    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)



import os
from json import dump, load

from .design_handler import GetPlayoffDesign, PostPlayoffDesign
from .data_handler import GetPlayoffData, PostPlayoffData
from .utility import MigrationLogger

from playoff import Playoff


class Export(object):
    """Base class that create a base dir and some parameters for exports
    classes
    """

    def __init__(self, game: Playoff):
        self.base_dir = "playoff-data"
        self.proj_path = os.getcwd() + "\\" + self.base_dir + "\\"

        if not os.path.isdir(self.proj_path):
            os.mkdir(self.base_dir)

        self.design_getter = GetPlayoffDesign(game)
        self.data_getter = GetPlayoffData(game)
        self.logger = MigrationLogger.get_instance()


class ExportRawDesign(Export):
    """Class that export raw design information from a game to a file .json"""

    def __init__(self, game: Playoff):
        super().__init__(game)

        self.dir_name = "design-raw"
        self.path = self.proj_path + self.dir_name + "\\"

        if not os.path.isdir(self.path):
            os.makedirs(self.path)

    def export_teams(self):
        """Saves raw teams design data from the original game in a .json
        file
        """
        self.logger.info("exporting raw teams design")

        with open(self.path + "teams_raw_design.json", "w+") as file:
            cloned_teams_design = []
            teams_design = self.design_getter.get_teams_design()

            for team in teams_design:
                self.logger.debug("exporting " + team['id'] + " design")

                single_team_design = self.design_getter\
                    .get_single_team_design(team['id'])

                cloned_teams_design.append(single_team_design)

            dump(cloned_teams_design, file, sort_keys=True, indent=4)

        self.logger.info("raw teams design exported")

    def export_metrics(self):
        """Saves raw metrics design data from the original game in a .json
        file
        """
        self.logger.info("exporting raw metrics design")

        with open(self.path + "metrics_raw_design.json", "w+") as file:
            cloned_metrics_design = []
            metrics_design_id = self.design_getter.get_metrics_design()

            for metric in metrics_design_id:
                self.logger.debug("exporting " + metric['id'] + " design")

                single_metric_design = self.design_getter\
                    .get_single_metric_design(metric['id'])

                cloned_metrics_design.append(single_metric_design)

            dump(cloned_metrics_design, file, sort_keys=True, indent=4)

        self.logger.info("raw metrics design exported")

    def export_actions(self):
        """Saves raw actions design data from the original game in a .json
        file
        """
        self.logger.info("exporting raw actions design")

        with open(self.path + "actions_raw_design.json", "w+") as file:
            cloned_actions_design = []
            actions_design = self.design_getter.get_actions_design()

            for action in actions_design:
                self.logger.debug("exporting " + action['id'] + " design")

                single_action_design = self.design_getter\
                    .get_single_action_design(action['id'])

                cloned_actions_design.append(single_action_design)

            dump(cloned_actions_design, file, sort_keys=True, indent=4)

        self.logger.info("raw actions design exported")

    def export_leaderboards(self):
        """Saves raw leaderboards design data from the original game in a .json
        file
        """
        self.logger.info("exporting raw leaderboards design")

        with open(self.path + "leaderboards_raw_design.json", "w+") as file:
            cloned_leaderboards_design = []
            leaderboards_id = self.design_getter.get_leaderboards_design()

            for board in leaderboards_id:
                self.logger.debug("exporting " + board['id'] + " design")

                single_design = self.design_getter\
                    .get_single_leaderboard_design(board['id'])

                cloned_leaderboards_design.append(single_design)

            dump(cloned_leaderboards_design, file, sort_keys=True, indent=4)

        self.logger.info("raw leaderboards design exported")

    def export_all_design(self):
        self.logger.info("exporting raw design")

        self.export_teams()
        self.export_metrics()
        self.export_actions()
        self.export_leaderboards()

        self.logger.info("raw design exported")


class ExportRawData(Export):
    """Class that export raw data information from a game to a file .json"""

    def __init__(self, game: Playoff):
        super().__init__(game)

        self.dir_name = "data-raw"
        self.path = self.proj_path + self.dir_name + "\\"

        if not os.path.isdir(self.path):
            os.makedirs(self.path)

    def export_teams(self):
        """Saves raw instances of all teams from the original game in a .json
        format
        """
        self.logger.info("exporting raw teams data")

        with open(self.path + "teams_raw.json", "w+") as file:
            cloned_teams_instances = []
            teams_by_id = self.data_getter.get_teams_by_id()

            for team in teams_by_id:
                self.logger.debug("exporting " + team + " data")

                team_instance_info = self.data_getter.get_team_info(team)

                cloned_teams_instances.append(team_instance_info)

            dump(cloned_teams_instances, file, sort_keys=True, indent=4)

        self.logger.info("raw teams data exported")

    def export_players(self):
        """Saves raw profile data of all players from the original game in a
        .json format
        """
        self.logger.info("exporting raw players data")

        with open(self.path + "players_raw.json", "w+") as file:
            cloned_players = []
            players_by_id = self.data_getter.get_players_by_id()

            for player in players_by_id:
                self.logger.debug("exporting " + player + " data")

                player_instance_info = self.data_getter\
                    .get_player_profile(player)

                cloned_players.append(player_instance_info)

            dump(cloned_players, file, sort_keys=True, indent=4)

        self.logger.info("raw players data exported")

    def export_players_feed(self):
        """Saves raw feed data of all players from the original game in a
        .json format
        """
        self.logger.info("exporting raw players feed data")

        with open(self.path + "players_feed_raw.json", "w+") as file:
            cloned_feed = {}
            players_id = self.data_getter.get_players_by_id()

            for player in players_id:
                self.logger.debug("exporting " + player + " feed data")

                player_feed = self.data_getter.get_player_feed(player)

                cloned_feed.update({player: player_feed})

            dump(cloned_feed, file, sort_keys=True, indent=4)

        self.logger.info("raw players feed data exported")

    def export_all_data(self):
        self.logger.info("exporting raw data")

        self.export_teams()
        self.export_players()
        self.export_players_feed()

        self.logger.info("raw data exported")


class ExportDesign(Export):

    def __init__(self, game: Playoff):
        super().__init__(game)

        self.dir_name = "design"
        self.path = self.proj_path + self.dir_name + "\\"

        if not os.path.isdir(self.path):
            os.makedirs(self.path)

    def export_teams(self):
        """ Create json file containing each team design of the original
        game
        """
        self.logger.info("exporting teams design")

        with open(self.path + "teams_design.json", "w+") as file:
            teams_design_clone = []
            teams_design = self.design_getter.get_teams_design()

            for team in teams_design:
                self.logger.debug("exporting " + team['id'] + " design")

                single_team_design = self.design_getter.get_single_team_design(
                    team['id'])

                team_design_data = {
                    'name': single_team_design['name'],
                    'id': single_team_design['id'],
                    'permissions': single_team_design['permissions'],
                    'creator_roles': single_team_design['creator_roles'],
                    'settings': single_team_design['settings'],
                    '_hues': single_team_design['_hues']
                }

                if 'description' in single_team_design.keys():
                    team_design_data.update(
                        {'description': single_team_design['description']})

                teams_design_clone.append(team_design_data)

            dump(teams_design_clone, file, sort_keys=True, indent=4)

        self.logger.info("teams design exported")

    def export_metrics(self):
        """ Create json file containing each metric design of the original
        game
        """
        self.logger.info("exporting metrics design")

        with open(self.path + "metric_design.json", "w+") as file:
            cloned_metrics = []
            metrics_design = self.design_getter.get_metrics_design()

            for metric in metrics_design:
                self.logger.debug("exporting " + metric['id'] + " design")

                single_metric_design = self.design_getter\
                    .get_single_metric_design(metric['id'])

                metric_design_data = {
                    "id": single_metric_design['id'],
                    "name": single_metric_design['name'],
                    "type": single_metric_design['type'],
                    "constraints": single_metric_design['constraints']
                }

                if "description" in single_metric_design.keys():
                    metric_design_data.update(
                        {"description": single_metric_design["description"]})

                cloned_metrics.append(metric_design_data)

            dump(cloned_metrics, file, sort_keys=True, indent=4)

        self.logger.info("metrics design exported")

    def export_actions(self):
        """ Create json file containing each action design of the original
        game
        """
        self.logger.info("exporting actions design")

        with open(self.path + "actions_design.json", "w+") as file:
            cloned_actions = []
            actions_design = self.design_getter.get_actions_design()

            for action in actions_design:
                self.logger.debug("exporting " + action['id'] + " design")

                single_action_design = self.design_getter\
                    .get_single_action_design(action['id'])

                cloned_actions.append(single_action_design)

            dump(cloned_actions, file, sort_keys=True, indent=4)

        self.logger.info("actions design exported")

    def export_leaderboards(self):
        """ Create json file containing each leaderboard design of the original
        game
        """
        self.logger.info("exporting leaderboards design")

        with open(self.path + "leaderboards_design.json", "w+") as file:
            cloned_leaderboards = []
            leaderboards = self.design_getter.get_leaderboards_design()

            for board in leaderboards:
                self.logger.debug("exporting " + board['id'] + " design")

                single_board = self.design_getter\
                    .get_single_leaderboard_design(board['id'])

                board_data = {
                    "id": single_board['id'],
                    "name": single_board['name'],
                    "entity_type": single_board['entity_type'],
                    "scope": single_board['scope'],
                    "metric": single_board['metric'],
                    "cycles": single_board['cycles']
                }

                if "description" in single_board.keys():
                    board_data.update(
                        {"description": single_board["description"]})

                cloned_leaderboards.append(board_data)

            dump(cloned_leaderboards, file, sort_keys=True, indent=4)

        self.logger.info("leaderboards design exported")

    def export_all_design(self):
        self.logger.info("exporting design")

        self.export_teams()
        self.export_metrics()
        self.export_actions()
        self.export_leaderboards()

        self.logger.info("design exported")


class ExportData(Export):

    def __init__(self, game: Playoff):
        super().__init__(game)

        self.dir_name = "data"
        self.path = self.proj_path + self.dir_name + "\\"

        if not os.path.isdir(self.path):
            os.makedirs(self.path)

    def export_teams(self):
        """ Create json file containing each team instance of the original
        game
        """
        self.logger.info("exporting teams data")

        with open(self.path + "teams.json", "w+") as file:
            cloned_teams_instances = []
            teams_by_id = self.data_getter.get_teams_by_id()

            for team in teams_by_id:
                self.logger.debug("exporting " + team + " data")

                team_data = self.data_getter.get_team_info(team)

                cloned_team_data = {
                    'id': team_data['id'],
                    'name': team_data['name'],
                    'access': team_data['access'],
                    'definition': team_data['definition']['id']
                }

                cloned_teams_instances.append(cloned_team_data)

            dump(cloned_teams_instances, file, sort_keys=True, indent=4)

        self.logger.info("teams data exported")

    def export_players(self):
        """ Create json file containing id and alias of each player of the
        original game
        """
        self.logger.info("exporting players data")

        with open(self.path + "players.json", "w+") as file:
            cloned_players = []
            players_by_id = self.data_getter.get_players_by_id()

            for player in players_by_id:
                self.logger.debug("exporting " + player + " data")

                player_data = self.data_getter.get_player_profile(player)

                cloned_player_data = {
                    'id': player_data['id'],
                    'alias': player_data['alias']
                }

                cloned_players.append(cloned_player_data)

            dump(cloned_players, file, sort_keys=True, indent=4)

        self.logger.info("raw players data exported")

    def export_players_in_team(self):
        """ Create json file containing the team of each player of the
        original game
        """
        self.logger.info("exporting players in team")

        with open(self.path + "players_in_team.json", "w+") as file:
            cloned_players_in_team = {}
            players_by_id = self.data_getter.get_players_by_id()

            for player in players_by_id:
                self.logger.debug("exporting teams of player " + player)

                player_profile = self.data_getter.get_player_profile(player)

                for team in player_profile['teams']:

                    cloned_team_player = {
                        "requested_roles": {
                            team['roles'][0]: True
                        },
                        "player_id": player
                    }

                    team_id = team['id']

                    if not (team_id in cloned_players_in_team.keys()):
                        cloned_players_in_team.update({team_id: []})
                        cloned_players_in_team[team_id]\
                            .append(cloned_team_player)
                    else:
                        cloned_players_in_team[team_id]\
                            .append(cloned_team_player)

            dump(cloned_players_in_team, file, sort_keys=True, indent=4)

        self.logger.info("players in team exported")

    def export_players_feed(self):
        """ Create json file containing the activity feed of each player of the
        original game
        """
        self.logger.info("exporting players feed data")

        with open(self.path + "players_feed.json", "w+") as file:
            cloned_players_feed = {}
            players_id = self.data_getter.get_players_by_id()

            for player in players_id:
                self.logger.debug("exporting " + player + " feed data")

                player_feed = self.data_getter.get_player_feed(player)

                feed_count = str(len(player_feed))
                index = 0

                for feed in player_feed:
                    player_single_feed = {}

                    if feed['event'] == 'action':
                        player_single_feed.update({"id": feed['action']['id']})
                        player_single_feed.update(
                            {"variables": feed['action']['vars']})
                        player_single_feed.update({"scopes": feed['scopes']})

                        if not (player in cloned_players_feed.keys()):
                            cloned_players_feed.update({player: []})
                            cloned_players_feed[player]\
                                .append(player_single_feed)
                        else:
                            cloned_players_feed[player]\
                                .append(player_single_feed)

                    index += 1
                    self.logger.debug("feed " + str(index) + " of " +
                                      feed_count + " exported")

            dump(cloned_players_feed, file, sort_keys=True, indent=4)

        self.logger.info("players feed data exported")

    def export_all_data(self):
        self.logger.info("exporting data")

        self.export_teams()
        self.export_players()
        self.export_players_in_team()
        self.export_players_feed()

        self.logger.info("data exported")


class Import(object):
    """Base class that saves base path and create an instance of a game for
    imports classes"""

    def __init__(self, game: Playoff):
        self.base_dir = "playoff-data"
        self.proj_path = os.getcwd() + "\\" + self.base_dir + "\\"

        if not os.path.isdir(self.proj_path):
            raise Exception("Directory doesn't exists")

        self.design_poster = PostPlayoffDesign(game)
        self.data_poster = PostPlayoffData(game)
        self.logger = MigrationLogger.get_instance()


class ImportDesign(Import):
    """Class that import design in a playoff game from a .json file"""

    def __init__(self, game: Playoff):
        super().__init__(game)

        self.dir_name = "design"
        self.path = self.proj_path + self.dir_name + "\\"

        if not os.path.isdir(self.path):
            raise Exception("Directory doesn't exists")

    def import_teams(self):
        """Import teams from .json file"""
        self.logger.info("importing teams design")

        with open(self.path + "teams_design.json", "r") as file:
            teams_design = load(file)

        for team in teams_design:
            self.logger.debug("importing " + team)

            self.design_poster.create_team_design(team)

        self.logger.info("teams design imported")

    def import_metrics(self):
        """Import metrics from .json file"""
        self.logger.info("importing metrics")

        with open(self.path + "metric_design.json", "r") as file:
            metrics_design = load(file)

        for metric in metrics_design:
            self.logger.debug("importing " + metric)

            self.design_poster.create_metric_design(metric)

        self.logger.info("teams imported")

    def import_actions(self):
        """Import actions from .json file"""
        self.logger.info("importing actions design")

        with open(self.path + "actions_design.json", "r") as file:
            actions_design = load(file)

        for action in actions_design:
            self.logger.debug("importing " + action)

            self.design_poster.create_action_design(action)

        self.logger.info("actions design imported")

    def import_leaderboards(self):
        """Import leaderboards from .json file"""
        self.logger.info("importing leaderboards design")

        with open(self.path + "leaderboards_design.json", "r") as file:
            leaderboards_design = load(file)

        for leaderboard in leaderboards_design:
            self.logger.debug("importing " + leaderboard)

            self.design_poster \
                .create_leaderboard_design(leaderboard)

        self.logger.info("leaderboards design imported")

    def import_all_design(self):
        self.logger.info("importing design")

        self.import_teams()
        self.import_metrics()
        self.import_actions()
        self.import_leaderboards()

        self.logger.info("design imported")


class ImportData(Import):
    """Class that import data in a playoff game from a .json file"""

    def __init__(self, game: Playoff):
        super().__init__(game)

        self.dir_name = "data"
        self.path = self.proj_path + self.dir_name + "\\"

        if not os.path.isdir(self.path):
            raise Exception("Directory doesn't exists")

    def import_teams(self):
        """Import teams from .json file"""
        self.logger.info("importing teams data")

        with open(self.path + "teams.json", "r") as file:
            teams = load(file)

        for team in teams:
            self.logger.debug("importing " + team)

            self.data_poster.create_team(team)

        self.logger.info("teams data imported")

    def import_players(self):
        """Import players from .json file"""
        self.logger.info("importing players data")

        with open(self.path + "players.json", "r") as file:
            players = load(file)

        for player in players:
            self.logger.debug("importing " + player)

            self.data_poster.create_player(player)

        self.logger.info("players data imported")

    def import_players_in_team(self):
        """Import players in team from .json file"""
        self.logger.info("importing players in team")

        with open(self.path + "players_in_team.json", "r") as file:
            players_in_team = load(file)

        for key, value in players_in_team.items():
            for player in value:
                self.logger.debug("importing " + player + " in " + key)

                self.data_poster.join_team(key, player)

        self.logger.info("players in team imported")

    def import_players_feed(self):
        """Import players feed from .json file"""
        self.logger.info("importing players feed")

        with open(self.path + "players_feed.json", "r") as file:
            players_feed = load(file)

        for key, value in players_feed.items():
            self.logger.debug("importing action for " + key)

            feed_count = str(len(value))
            index = 0

            for feed in value:
                data = {
                    "scopes": feed["scopes"],
                    "variables": feed["variables"]
                }
                self.data_poster.take_action(feed['id'], {"player_id": key},
                                             data)

                index += 1
                self.logger.debug("imported action " + str(index) + " of " +
                                  feed_count + " feed")

    def import_all_data(self):
        self.logger.info("importing data")

        self.import_teams()
        self.import_players()
        self.import_players_in_team()
        self.import_players_feed()

        self.logger.info("data imported")
